package com.yihaomen.mosquitto_starter.handler;

import java.net.URISyntaxException;
import java.util.concurrent.atomic.AtomicBoolean;

import org.fusesource.mqtt.client.Callback;
import org.fusesource.mqtt.client.Future;
import org.fusesource.mqtt.client.FutureConnection;
import org.fusesource.mqtt.client.MQTT;
import org.fusesource.mqtt.client.Message;
import org.fusesource.mqtt.client.QoS;
import org.fusesource.mqtt.client.Topic;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;

import com.yihaomen.mosquitto_starter.MosquittoProperties;

public class MosquittoMsgReceiver {
	
    private static Logger logger = LoggerFactory.getLogger(MosquittoMsgPublisher.class);	
	
	private final AtomicBoolean running = new AtomicBoolean(true);		
	
	@Autowired
	private MosquittoProperties mosquittoProperties;
	
	public void run(MessageHandler messageHandler) {
		
		MQTT mqtt = new MQTT();	
				
        try {
			mqtt.setHost("tcp://" + mosquittoProperties.getAddress());
			if (!StringUtils.isEmpty(mosquittoProperties.getUser())) {
				mqtt.setUserName(mosquittoProperties.getUser());
				mqtt.setPassword(mosquittoProperties.getPassword());			
			}
		} catch (URISyntaxException e) {
			
		}
        
        FutureConnection connection = mqtt.futureConnection();       
        connection.connect();
        connection.subscribe(new Topic[] { new Topic( mosquittoProperties.getTopic(), QoS.AT_LEAST_ONCE) });
        Future<Message> receive;
        
		while (running.get()) {
			receive = connection.receive();
			receive.then(new Callback<Message>() {
				@Override
				public void onSuccess(Message message) {
					try {
						String topic = message.getTopic();
						String msg = new String(message.getPayload(), "utf-8");
						messageHandler.handleMessage(msg, topic);
					} catch (Exception e) {
						logger.error("==>error: " + e.getMessage());
					}					
					message.ack();
				}

				@Override
				public void onFailure(Throwable value) {
					logger.error("==>{} get message error.", Thread.currentThread().getName());
				}
			});
			
			try {
                receive.await();
            } catch (Exception e) {
            	e.printStackTrace();
                logger.error("==>future connection error: {}", e.getMessage());
            }			
		}		
		connection.disconnect();		
	}

}
